#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightColor;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform float expConst;
uniform sampler2D diffuseTexture;
uniform sampler2D normalMap;
uniform samplerCube depthMap;

uniform int drawNormals;

uniform float far_plane;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

// array of offset direction for sampling
vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

float ShadowCalculation(vec3 fragPos)
{
	vec3 lightPos = pointL.position;
    vec3 fragToLight = fragPos - lightPos;
    float currentDepth = length(fragToLight);
    float shadow = 0.0;
	float bias = max(0.05 * (1.0 - dot(vert.normal_world, fragToLight)), 0.005);
    int samples = 20;
    float viewDistance = length(camera_world - fragPos);
    float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
    for(int i = 0; i < samples; ++i)
    {
        float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
        closestDepth *= far_plane;   // undo mapping [0;1]
        if(currentDepth - bias < closestDepth)
            shadow += 1.0;
    }
    shadow /= float(samples);
        
    // debug (to visualize depth cubemap)
    // color = vec4(vec3(closestDepth / far_plane), 1.0);    
        
    return shadow;
}

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 vertexNormal = normalize(vert.normal_world);
	vec3 n = vertexNormal;
	if (drawNormals == 1) {
		vec3 normalFromMap = texture(normalMap, vert.uv).rgb;
		normalFromMap = normalize(normalFromMap * 2.0 - 1.0);
		n = normalFromMap;
		vec3 tangent = normalize(cross(vertexNormal, vec3(0, 0, 1)));
		vec3 bitangent = normalize(cross(vertexNormal, tangent));
		mat3 TBN = mat3(tangent, bitangent, vertexNormal);
		n = normalize(TBN * normalFromMap);
	}
	vec3 v = normalize(camera_world - vert.position_world);

	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x * expConst, 1); // ambient
	
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);

	// add shadow
	float shadow = ShadowCalculation(vert.position_world); 
	color.rgb *= shadow;
	
    float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 1.0)
        brightColor = vec4(color.rgb, 1.0);
    else
        brightColor = vec4(0.0, 0.0, 0.0, 1.0);


	//color.rgb = (n + vec3(1)) / 2.0; //showing normals for debugging
}

